/*
 * Copyright (c) 1991-2023 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and
 * its documentation is hereby granted, provided that this copyright
 * notice and license appear on all copies of the software.
 * 
 * STEP TOOLS MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
 * SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. STEP TOOLS
 * SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES.
 *
 * Author: David Loffredo (loffredo@steptools.com)
 */


#include <stp_schema.h>
#include "toolpath.h"

/* Machining tools are a step action_resource, which means they
 * contain a list of things that use them.  Because of this, we can
 * not write the tool data until we are done with the workplan.
 */

void ToolpathMaker::Cutter(
    double diameter, 
    double corner_radius, 
    double corner_radius_horizontal, 
    double corner_radius_vertical, 
    double tip_outer_angle, 
    double taper_angle, 
    double height
    )
{
    // Create a new cutter record for these parameters.
    tooldef * t = new tooldef;
    t-> diameter = diameter;
    t-> corner_radius = corner_radius;
    t-> corner_radius_horizontal = corner_radius_horizontal;
    t-> corner_radius_vertical = corner_radius_vertical;
    t-> tip_outer_angle = tip_outer_angle;
    t-> taper_angle = taper_angle;
    t-> height = height;

    t-> tool_num = 0;
    t-> next_tool = all_tools;
    all_tools = t;

    // do not change current_tool, only load tool does that.
}


void ToolpathMaker::LoadTool (unsigned num)
{
    // Tool change starts a new workingstep
    end_workingstep();
    
    // Loads a particular tool number.  Assigns the number to the last
    // defined cutter if the tool number does not already exist.
    //
    tooldef * t = all_tools;
    while (t) {
	if (t-> tool_num == num) {
	    current_tool = t;
	    return;
	}
	t = t-> next_tool;
    }

    // No such tool, assign one possibly creating it, afterwards the
    // latest tool will be in all_tools
    //
    SELCTLTool (num);
    current_tool = all_tools;
}


/* assigns tool number to last defined cutter */
void ToolpathMaker::SELCTLTool(unsigned num)
{
    // Error!  Zero is not a valid tool number
    if (!num) return;

    // Use the previous definition for existing tool number
    tooldef * t = all_tools;
    while (t) {
	if (t-> tool_num == num) return;
	t = t-> next_tool;
    }

    // If no new tool has been defined, just copy the last one
    t = all_tools;

    if (!all_tools) {
	// no tool defined, just make one 
	Cutter (0,0,0,0,0,0,0);
    } 
    else if (all_tools-> tool_num != 0) {
	// last cutter already has a number, make a copy 
	Cutter (all_tools-> diameter,
		all_tools-> corner_radius,
		all_tools-> corner_radius_horizontal,
		all_tools-> corner_radius_vertical,
		all_tools-> tip_outer_angle,
		all_tools-> taper_angle,
		all_tools-> height
	    );
    }

    // Give the last tool the number
    all_tools-> tool_num = num;
}


void ToolpathMaker::make_all_tools()
{
    // Go through all of the tool definitions and create them.
    //
    stp_action_resource_type * artyp = pnewIn(f_design) 
	stp_action_resource_type;
    artyp-> name ("milling cutting tool");


    tooldef * t = all_tools;
    while (t) {
	unsigned i,sz;

	stp_machining_tool * mt = pnewIn(f_design) stp_machining_tool;
	mt-> description ("user defined milling tool");
	mt-> kind (artyp);

	if (t-> tool_num) {
	    char buf[50];
	    sprintf (buf, "%u", t-> tool_num);
	    mt-> name (buf);
	}
	else
	    mt-> name ("unused tool");

	for (i=0,sz=t->users.size(); i<sz; i++) 
	{
	    stp_supported_item * si = pnewIn(f_design) stp_supported_item;
	    si-> _action_method ((stp_machining_operation *) t->users[i]);
	    mt-> usage()-> add (si);
	}

	// Tie all of the parameters together into a tool body
	stp_representation * rep = pnewIn(f_design)
	    stp_machining_tool_body_representation;
	rep-> name ("");
	rep-> context_of_items (make_simple_context());


	// Link the rep back to the machining tool
	stp_resource_property * prop = pnewIn(f_design) stp_resource_property;
	prop-> name ("tool body");
	prop-> description ("");
	prop-> resource (
	    pnewIn(f_design) stp_characterized_resource_definition
	    );

	prop-> resource()-> _action_resource(mt);

	
	stp_resource_property_representation * rpr = pnewIn(f_design)
	    stp_resource_property_representation;
	rpr-> name ("");
	rpr-> description ("");
	rpr-> property (prop);
	rpr-> representation (rep);


	rep-> items()-> add (
	    make_length_measure("effective cutting diameter", t-> diameter)
	    );


	// Always write the diameter and height, but omit these if
	// they are zero
	if (STEPNC_FLOAT_NOT_ZERO(t-> corner_radius))
	    rep-> items()-> add (
		make_length_measure (
		    "corner radius", 
		    t-> corner_radius)
		);

	if (STEPNC_FLOAT_NOT_ZERO(t-> corner_radius_horizontal))
	    rep-> items()-> add (
		make_length_measure(
		    "corner radius center horizontal", 
		    t-> corner_radius_horizontal)
		);

	if (STEPNC_FLOAT_NOT_ZERO(t-> corner_radius_vertical))
	    rep-> items()-> add (
		make_length_measure(
		    "corner radius center vertical", 
		    t-> corner_radius_vertical)
		);

	if (STEPNC_FLOAT_NOT_ZERO(t-> taper_angle))
	    rep-> items()-> add (
		make_angle_measure(
		    "taper angle", 
		    t-> taper_angle)
		);

	if (STEPNC_FLOAT_NOT_ZERO(t-> tip_outer_angle))
	    rep-> items()-> add (
		make_angle_measure(
		    "tip outer angle", 
		    t-> tip_outer_angle)
		);

	// The APT height parameter specifies the cutting portion of the
	// tool.  The milling tool base type allows specification of both
	// maximum_depth_of_cut and overall_assembly_length.
	//
	rep-> items()-> add (
	    make_length_measure(
		"maximum depth of cut", 
		t-> height)
	    );


	t = t-> next_tool;
    }
}
