/*
 * Copyright (c) 1991-2023 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and
 * its documentation is hereby granted, provided that this copyright
 * notice and license appear on all copies of the software.
 * 
 * STEP TOOLS MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
 * SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. STEP TOOLS
 * SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES.
 *
 * Author: David Loffredo (loffredo@steptools.com)
 */


#include <stp_schema.h>
#include <stix_asm.h>

#include "toolpath.h"

ToolpathMaker::ToolpathMaker() 
{
    f_design = 0;
    all_techs = 0;
    all_mfuncs = 0;
    all_tools = 0;

    Reset();  	/* clear all of the shared ids too */
    Inches(); 	/* Set default units */
}

ToolpathMaker::~ToolpathMaker() 
{
    Reset();  // close file and delete any state info
}



void ToolpathMaker::Reset()
{
    int i;

    // make sure file is closed if we had an open one 
    f_design = 0;

    /* delete all techs */
    techdef * t = all_techs;
    while (t) {
	techdef * dt = t;
	t = t-> next_tech;
	delete dt;
    }
    is_rapid = 0;
    current_speed = 0;
    current_feed = 0;
    current_tech = 0;
    all_techs = 0;

    /* delete all mfuncs */
    mfuncdef * mf = all_mfuncs;
    while (mf) {
	mfuncdef * dt = mf;
	mf = mf-> next_mfunc;
	delete dt;
    }
    current_coolant = 0;
    current_mist = 0;
    current_mfunc = 0;
    all_mfuncs = 0;

    /* delete all tool defs */
    tooldef * tl = all_tools;
    while (tl) {
	tooldef * dt = tl;
	tl = tl-> next_tool;
	delete dt;
    }
    current_tool = 0;
    all_tools = 0;

    /* clear the curve states */
    for (i=0; i<3; i++) loc.last_point[i]=0;
    for (i=0; i<3; i++) axis.last_point[i]=0;

    loc.last_point_id = 0;
    axis.last_point_id = 0;

    current_zdir = 0;
    current_xdir = 0;

    // workplan state information
    ws_count = 0;
    tp_count = 0;
    current_op = 0;
    current_tp = 0;
    current_tpidx = 0;

    current_project_pd = 0;
    current_workpiece_pds = 0;
    current_feature_mfp =0;

    ctx_product = 0;
    ctx_pdef = 0;

    plan_ids.empty();
    plan_idxs.empty();
}



void ToolpathMaker::Design (
    RoseDesign * d
    )
{
    if (f_design) Done();

    Reset();
    f_design = d;


    // CONTEXTS -- We need some context objects for the project and
    // workpiece.  Create and save for later.
    //
    stp_application_context * apc = pnewIn(f_design) stp_application_context;
    apc-> application("Application protocol for the exchange of CNC data");

    stp_application_protocol_definition * apd = pnewIn(f_design)
	stp_application_protocol_definition;

    apd-> status ("international standard");
    apd-> application_interpreted_model_schema_name ("integrated_cnc_schema");
    apd-> application_protocol_year (2007);
    apd-> application (apc);


    ctx_product = pnewIn(f_design) stp_product_context;
    ctx_product-> name("CNC Machining");
    ctx_product-> discipline_type("manufacturing");
    ctx_product-> frame_of_reference (apc);

    ctx_pdef = pnewIn(f_design) stp_product_definition_context;
    ctx_pdef-> name("CNC Machining");
    ctx_pdef-> life_cycle_stage("manufacturing");
    ctx_pdef-> frame_of_reference (apc);


    // PROJECT -- A STEP-NC file only has a single project and it
    // contains the reference to the main workplan.
    //
    stp_machining_project * proj = pnewIn(f_design) stp_machining_project;
    proj-> id ("unnamed project");
    proj-> frame_of_reference()-> add (ctx_product);

    stp_product_definition_formation * pdf = pnewIn(f_design)
	stp_product_definition_formation;

    pdf-> id("1.0");
    pdf-> description("");
    pdf-> of_product (proj);

    current_project_pd = pnewIn(f_design) stp_product_definition;
    current_project_pd-> id("");
    current_project_pd-> description("");
    current_project_pd-> formation (pdf);
    current_project_pd-> frame_of_reference (ctx_pdef);


    // ------------------------------
    // MAIN WORKPLAN -- Create the main workplan.  This is associated
    // with the machining project so that we know where to start.
    //
    make_workplan ("main workplan");
}
	

void ToolpathMaker::Done() 
{
    if (!f_design) return;

    // close out any toolpath curves and outstanding tools;
    end_move();

    // force a workpiece if we have not written one yet;
    if (!current_workpiece_pds) PartNo("unknown workpiece");

    make_all_tools();
}




void ToolpathMaker::PartNo(
    const char *wp_name	
    )
{
    // Write a workpiece and associate it with the machining project.
    // This only creates a workpiece the first time it is called and
    // ignores subsequent calls.
    //
    if (!f_design || current_workpiece_pds) return;

    stp_product * prod = pnewIn(f_design) stp_product;
    prod-> id (wp_name? wp_name: "unnamed workpiece");
    prod-> name ("AP238 CC1");
    prod-> frame_of_reference()-> add(ctx_product);


    stp_product_definition_formation * pdf = pnewIn(f_design)
	stp_product_definition_formation;
    pdf-> id("1.0");
    pdf-> description("workpiece");
    pdf-> of_product(prod);

    stp_product_definition * pd = pnewIn(f_design) stp_product_definition;
    pd-> id("workpiece");
    pd-> description("");
    pd-> formation(pdf);
    pd-> frame_of_reference(ctx_pdef);

    // Associate with the project
    stp_machining_project_workpiece_relationship * rel = pnewIn(f_design) 
	stp_machining_project_workpiece_relationship;
    rel-> id("");
    rel-> name("workpiece");
    rel-> description("");
    stix_put_relating_pdef(rel, current_project_pd);
    stix_put_related_pdef(rel, pd);

    // Give the workpiece a shape property to attach features and such
    current_workpiece_pds = pnewIn(f_design) stp_product_definition_shape;
    current_workpiece_pds-> name("");
    current_workpiece_pds-> description("");
    current_workpiece_pds-> definition(
	pnewIn(f_design) stp_characterized_definition
	);
    rose_put_nested_object (current_workpiece_pds-> definition(), pd);
}



void ToolpathMaker::PPrint(
	const char * label, 
	const char * message
	)
{
    if (!f_design) return;

    end_workingstep();

    stp_machining_nc_function * id = pnewIn(f_design) 
	stp_machining_nc_function;

    id-> name(label? label: "");
    id-> description("display message");
    id-> consequence("");
    id-> purpose("");

    make_action_property ("message text", id, make_string_rep (message));

    add_to_workplan (id);
}
    

void ToolpathMaker::GoHome (const char * label)
{
    if (!f_design) return;

    // this is a type of workingstep, so it ends the previous one
    end_workingstep();

    stp_machining_rapid_movement * id = pnewIn(f_design) 
	stp_machining_rapid_movement;

    id-> name(label? label: "");
    id-> description("return home");
    id-> consequence("");
    id-> purpose("");

    // Associate the toolpath with the current operation using a
    // sequence relationship object
    //
    add_to_workplan(id);

    loc.last_point_id = 0;
    axis.last_point_id = 0;
}


void ToolpathMaker::Stop (const char * label)
{
    if (!f_design) return;

    end_workingstep();

    stp_machining_nc_function * id = pnewIn(f_design) 
	stp_machining_nc_function;

    id-> name(label? label: "");
    id-> description("program stop");
    id-> consequence("");
    id-> purpose("");

    add_to_workplan (id);
}


void ToolpathMaker::Connector()
{
    if (!f_design) return;

    // Close out any toolpath
    end_move();

    // Make a new workingstep if needed
    if (!current_op) make_workingstep("");


    stp_machining_toolpath * tp = pnewIn(f_design) stp_machining_toolpath;
    tp-> name("");
    tp-> description("connect direct");
    tp-> consequence("");
    tp-> purpose("");

    // Some common properties for the toolpaths
    //
    make_action_property ("trajectory type", tp, make_string_rep ("connect"));
    make_action_property ("priority", tp, make_string_rep ("required"));

    // Associate the toolpath with the current operation using a
    // sequence relationship object
    //
    add_to_workingstep(tp);

    loc.last_point_id = 0;
    axis.last_point_id = 0;
}


