/*
 * Copyright (c) 1991-2023 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and
 * its documentation is hereby granted, provided that this copyright
 * notice and license appear on all copies of the software.
 * 
 * STEP TOOLS MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
 * SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. STEP TOOLS
 * SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES.
 *
 * Author: David Loffredo (loffredo@steptools.com)
 */

#include <stp_schema.h>
#include "toolpath.h"

void ToolpathMaker::Feedrate(
    double feed
    )
{
    if (!f_design) return;
    if (STEPNC_FLOAT_EQUAL (current_feed, feed) && !is_rapid)
	return;

    end_move();	    // Feed change forces a new toolpath

    current_feed = feed;
    is_rapid = 0;
}


void ToolpathMaker::Rapid()
{
    if (!f_design) return;
    if (is_rapid) return;

    end_move();	    // Feed change forces a new toolpath

    is_rapid = 1;
}


void ToolpathMaker::SpindleSpeed(
    double speed
    )
{
    if (!f_design) return;
    if (STEPNC_FLOAT_EQUAL (current_speed, speed))
	return;

    // As a matter of policy, You can have a spindle change either
    // start a new toolpath or a new workinngstep
    //
    // end_move();	    // Spindle change forces a new toolpath
    end_workingstep();	    // Spindle change forces a new workingstep

    current_speed = speed;
}


// The speeds and feeds are handled as properties on a
// machining_technology object.  Rapid is handled as a speed profile
// modifier on the toolpath.  Other process parameters can go on the
// technology too, but we do not bother with them yet.
// 
stp_machining_technology * ToolpathMaker::make_technology()
{
    // No change in feeds/speeds?
    if (current_tech &&
	STEPNC_FLOAT_EQUAL (current_feed,  current_tech-> feed) &&
	STEPNC_FLOAT_EQUAL (current_speed, current_tech-> speed))
    {
	return current_tech-> tech_id;
    }
    
    // Look for an existing technology record for this particular
    // feed/speed combination.
    techdef * t = all_techs;
    while (t) {
	if (STEPNC_FLOAT_EQUAL (current_feed,  t-> feed) &&
	    STEPNC_FLOAT_EQUAL (current_speed, t-> speed))
	{
	    current_tech = t;
	    return t-> tech_id;
	}
	t = t-> next_tech;
    }

    // No match, create a new technology for this feed/speed
    t = new techdef;
    t-> feed  = current_feed;
    t-> speed = current_speed;
    t-> next_tech = all_techs;
    all_techs = t;
    current_tech = t;


    t-> tech_id = pnewIn(f_design) stp_machining_technology;
    t-> tech_id-> name ("");
    t-> tech_id-> description ("milling");
    t-> tech_id-> consequence ("");
    t-> tech_id-> purpose ("");

    /* Create properties for the spindle speed and feed values */

    stp_representation * rep = pnewIn(f_design) 
	stp_machining_spindle_speed_representation;

    rep-> name ("spindle speed");
    rep-> context_of_items (make_simple_context());
    rep-> items()-> add (
	make_spindle_measure ("rotational speed", current_speed)
	);

    make_action_property ("spindle", t-> tech_id, rep);


    rep = pnewIn(f_design) stp_machining_feed_speed_representation;
    rep-> name ("feed speed");
    rep-> context_of_items (make_simple_context());
    rep-> items()-> add (
	make_feed_measure ("feed speed", current_feed)
	);

    make_action_property ("feedrate", t-> tech_id, rep);

    return t-> tech_id;
}

